<!DOCTYPE html>
<html>
<head>
        <title>Address Formset</title>
</head>
<body>

<h1>Address Formset</h1>
<form action="." method="post">

{{ formset }}


<input type="submit" value="Submit" />

</form>

</body>
</html>